﻿


Public Class SGP_OrganicTest

    'Add a new mesh to current space, setting it to a Sphere primitive. No smoothing.
    <CommandMethod("AddSphere")> _
    Public Sub addSphere()

        Dim doc As Document = Application.DocumentManager.MdiActiveDocument
        Dim db As Database = doc.Database

        Using tr As Transaction = doc.TransactionManager.StartTransaction

            Dim btr As BlockTableRecord = tr.GetObject(db.CurrentSpaceId, Autodesk.AutoCAD.DatabaseServices.OpenMode.ForWrite, False)
            Dim mySubD As New SubDMesh
            mySubD.SetSphere(20, 8, 8, 0)
            mySubD.SetDatabaseDefaults()
            btr.AppendEntity(mySubD)
            tr.AddNewlyCreatedDBObject(mySubD, True)

            tr.Commit()
        End Using

    End Sub













#Region "createCasing helper functions"

    'Add creases to user selected faces
    'Returns true if successful
    Private Function addCreases(ByVal meshId As ObjectId) As Boolean
        Dim doc As Document = Application.DocumentManager.MdiActiveDocument
        Dim selOpts As New PromptSelectionOptions
        selOpts.ForceSubSelections = True
        selOpts.AllowDuplicates = False
        selOpts.MessageForAdding = "Select edges to crease:"
        Dim selRes As PromptSelectionResult = doc.Editor.GetSelection(selOpts)

        'If user didn't make valid selection, then we assume they want to end the command
        If selRes.Status <> PromptStatus.OK Then Return False

        Dim SS As SelectionSet = selRes.Value


        'They may have picked more than one object - we need the subobjects of the object with the right ObjectId
        Dim obj As SelectedObject = Nothing
        For Each o As SelectedObject In SS
            If o.ObjectId = meshId Then
                obj = o
            End If
        Next
        If obj Is Nothing Then Return False 'If they picked on the wrong object, then assume user wants to end command

        'We got to here so selection was valid

        'This will store selected edges
        Dim creasePathCollection As New System.Collections.Generic.List(Of FullSubentityPath)

        'Start transaction - we're editing the mesh
        Using tr As Transaction = doc.TransactionManager.StartTransaction

            Dim mySubD As SubDMesh = tr.GetObject(obj.ObjectId, Autodesk.AutoCAD.DatabaseServices.OpenMode.ForWrite, False)

            'Add all selected subentities to the collection
            Dim subEnts() As SelectedSubObject = obj.GetSubentities()
            For Each subEnt As SelectedSubObject In obj.GetSubentities
                If subEnt.FullSubentityPath.SubentId.Type = SubentityType.Edge Then
                    creasePathCollection.Add(subEnt.FullSubentityPath)
                End If
            Next

            'Add infinite creases to all those edges
            mySubD.SetCrease(creasePathCollection.ToArray, -1) '-1 means 'always' crease

            'Our work here is done
            tr.Commit()

        End Using

        Return True

    End Function

    'Extrude each mesh face by a random amount, up to a user specified maximum.
    'Assume meshId is ObjectId of valid SubDMesh.
    'Returns true if successful
    Private Function extrudeFaces(ByVal meshId As ObjectId) As Boolean
        Dim doc As Document = Application.DocumentManager.MdiActiveDocument

        'Now we want to stretch a few faces to add a bit of randomness
        'Better just stretch outwards

        'Prompt for max random bump (0.02 works well for sample drawing)
        Dim dblOpts As New PromptDoubleOptions(vbCrLf + "Specify bump size as fraction of bounds:")
        Dim dblRes As PromptDoubleResult = doc.Editor.GetDouble(dblOpts)

        'If user cancels value entry, then we assume they want to cancel command
        If dblRes.Status <> PromptStatus.OK Then Return False


        Dim extrudelen As Double = dblRes.Value
        Dim selOpts As New PromptSelectionOptions
        selOpts.ForceSubSelections = True
        selOpts.MessageForAdding = "Select faces to bump:"
        Dim selRes As PromptSelectionResult = doc.Editor.GetSelection(selOpts)

        'If user cancels selection, then we assume they want to end command any surfaces
        If selRes.Status <> PromptStatus.OK Then Return False

        Dim SS As SelectionSet = selRes.Value

        'They may have picked more than one object - we need the subobjects of the object with the right ObjectId
        Dim obj As SelectedObject = Nothing
        For Each o As SelectedObject In SS
            If o.ObjectId = meshId Then
                obj = o
            End If
        Next
        If obj Is Nothing Then Return False 'If they picked on the wrong object, then assume they want to end command

        'We got to here so selection was valid

        'Stores faces we selected for random extrusion
        Dim bumpPathCollection As New System.Collections.Generic.List(Of FullSubentityPath)

        'Start transaction - we're editing the mesh
        Using tr As Transaction = doc.TransactionManager.StartTransaction


            Dim mySubD As SubDMesh = tr.GetObject(obj.ObjectId, Autodesk.AutoCAD.DatabaseServices.OpenMode.ForWrite, False)
            Dim subEnts() As SelectedSubObject = obj.GetSubentities()

            'We'll be extruding faces by a random amount, so instantiate random number generator here ready to use inside loop
            Dim rnd As New Random

            'Process each selected face in turn
            For Each subEnt As SelectedSubObject In obj.GetSubentities
                'Is it a face?
                If subEnt.FullSubentityPath.SubentId.Type = SubentityType.Face Then
                    Dim faces(0) As FullSubentityPath
                    faces(0) = subEnt.FullSubentityPath

                    'Find normal for this face
                    Dim fPlane As Plane = mySubD.GetFacePlane(faces(0).SubentId)
                    Dim norm As Vector3d = fPlane.Normal

                    'Get length of diagonal across bounds to use in calculating bump scale
                    Dim ext As Extents3d = mySubD.Bounds
                    Dim vec As Vector3d = (ext.MaxPoint - ext.MinPoint)

                    'Extrude face along its normal by a length between 0 and the bump scale
                    Dim mat As Matrix3d = Matrix3d.Displacement(0.5 * norm * extrudelen * rnd.NextDouble * vec.Length)
                    mySubD.TransformSubentityPathsBy(faces, mat)

                End If
            Next

            'Our work here is done
            tr.Commit()
        End Using

        Return True

    End Function

    'Set random color for each mesh face.
    'Assume meshId is valid ObjectId of a SubDMesh
    Private Sub colorFaces(ByVal meshId)
        Dim doc As Document = Application.DocumentManager.MdiActiveDocument

        'Start transaction - we're editing the mesh
        Using tr As Transaction = doc.TransactionManager.StartTransaction
            Dim rnd As New Random

            Dim mySubD As SubDMesh = tr.GetObject(meshId, Autodesk.AutoCAD.DatabaseServices.OpenMode.ForWrite, False)

            'Iterate all faces and set to a random color
            For i As Integer = 0 To mySubD.NumberOfFaces - 1
                Dim subEntId As New SubentityId(SubentityType.Face, i)
                Dim RGB(2) As Byte
                rnd.NextBytes(RGB)
                Dim col As Autodesk.AutoCAD.Colors.Color = Autodesk.AutoCAD.Colors.Color.FromRgb(RGB(0), RGB(1), RGB(2))
                mySubD.SetSubentColor(subEntId, col)
            Next

            'Our work here is done
            tr.Commit()
        End Using

    End Sub


    'Make clone of solid, inflate it by a fraction of its extents, create a mesh from the clone, erase the clone.
    'Assume solidId is ObjectID of valid Solid3d
    Private Function cloneAndInflateSolid(ByVal solidId As ObjectId) As ObjectId
        'First copy the solid
        Dim doc As Document = Application.DocumentManager.MdiActiveDocument
        Dim db As Database = doc.Database
        Dim newId As ObjectId

        'Inflate solid by user specified fraction (0.05 works well with sample drawing)
        Dim dblOpts As New PromptDoubleOptions(vbCrLf + "Specify inflation as fraction of bounds:")
        Dim dblRes As PromptDoubleResult = doc.Editor.GetDouble(dblOpts)

        'Return if something went wrong.
        If dblRes.Status <> PromptStatus.OK Then Return ObjectId.Null

        Dim extrudeFrac As Double = dblRes.Value
        Using tr As Transaction = doc.TransactionManager.StartTransaction
            Dim ids As New ObjectIdCollection
            Dim IdMap As New IdMapping
            ids.Add(solidId)
            db.DeepCloneObjects(ids, db.CurrentSpaceId, IdMap, False)

            newId = IdMap.Lookup(solidId).Value

            'We now have the ObjectID of the newly cloned solid in newId

            'Inflate by length of diagonal vector across bounds multiplied by inflation fraction
            Dim mySolid As Solid3d = tr.GetObject(newId, Autodesk.AutoCAD.DatabaseServices.OpenMode.ForWrite, False)
            Dim ext As Extents3d = mySolid.Bounds
            Dim vec As Vector3d = (ext.MaxPoint - ext.MinPoint)
            mySolid.OffsetBody(vec.Length * extrudeFrac)

            'Define params governing mesh generation algorithm
            '(See ObjectARX helpfiles for explanation of params)
            Dim myFaceterData As New MeshFaceterData(0.01 * vec.Length, 40 * Math.PI / 180, 2, 2, 15, 5, 5, 0)

            'Create new mesh from solid (smoothing level 1)
            Dim meshData As MeshDataCollection = SubDMesh.GetObjectMesh(mySolid, myFaceterData)
            Dim myMesh As New SubDMesh
            myMesh.SetSubDMesh(meshData.VertexArray, meshData.FaceArray, 1)

            'Add mesh to database. 
            myMesh.SetDatabaseDefaults()
            Dim btr As BlockTableRecord = tr.GetObject(db.CurrentSpaceId, Autodesk.AutoCAD.DatabaseServices.OpenMode.ForWrite)
            btr.AppendEntity(myMesh)
            tr.AddNewlyCreatedDBObject(myMesh, True)
            'Erase cloned solid
            mySolid.Erase()
            newId = myMesh.ObjectId

            'Our work here is done
            tr.Commit()

        End Using

        Return newId
    End Function








    'Convert a SubDMesh object to solid
    'meshId is assumed to be valid ObjectId of a SubDMesh
    Private Function convertToSolid(ByVal meshId As ObjectId) As ObjectId
        Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
        Dim newObjId As ObjectId

        Using tr As Transaction = db.TransactionManager.StartTransaction

            Dim mySubD As SubDMesh = tr.GetObject(meshId, Autodesk.AutoCAD.DatabaseServices.OpenMode.ForWrite)

            'Note - This function will throw an exception if it can't create a solid from the mesh.
            'We're not trapping the exception, so it wil create an exception dialog if the mesh is too complex to solidify.
            Dim mySolid As Solid3d = mySubD.ConvertToSolid(True, True)

            Dim BTR As BlockTableRecord = tr.GetObject(db.CurrentSpaceId, Autodesk.AutoCAD.DatabaseServices.OpenMode.ForWrite, False)
            newObjId = BTR.AppendEntity(mySolid)
            tr.AddNewlyCreatedDBObject(mySolid, True)

            tr.Commit()

        End Using

        Return newObjId

    End Function



    'Translates entity along x-axis by (factor)x(entity's extent along  x-axis)
    Private Sub translateEntity(ByVal objId As ObjectId, ByVal factor As Double)

        Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database

        Using tr As Transaction = db.TransactionManager.StartTransaction

            Dim ent As Entity = tr.GetObject(objId, Autodesk.AutoCAD.DatabaseServices.OpenMode.ForWrite)

            Dim ext As Extents3d = ent.Bounds
            Dim pt1 As New Point3d(ext.MaxPoint.X, ext.MinPoint.Y, ext.MinPoint.Z)

            Dim transVec As Vector3d = factor * (pt1 - ext.MinPoint)

            Dim mat As Matrix3d = Matrix3d.Displacement(transVec)
            ent.TransformBy(mat)

            tr.Commit()
        End Using


    End Sub


#End Region


    'Creates a SubDMesh based on an 'inflated' clone of selected solid. 
    'User then gets to add creases and select which faces will be given a random extrusion
    'Colors each face of mesh and then converts to smooth solid.
    'Note - Converting to solid will throw exception if mesh is too complex to create the solid.
    'We're not trapping that exception in our code.
    <CommandMethod("CreateCasing")> _
    Public Sub createCasing()

        Dim meshId As ObjectId

        'First select the solid (just one)
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
        Dim opts As New PromptEntityOptions(vbCrLf + "Select Solid:")
        opts.SetRejectMessage(vbCrLf & "That's not a solid!" & vbCrLf)
        opts.AddAllowedClass(GetType(Solid3d), False)
        Dim res As PromptEntityResult = ed.GetEntity(opts)

        If res.Status <> PromptStatus.OK Then Exit Sub

        meshId = cloneAndInflateSolid(res.ObjectId)

        'If the above call returned a Null ObjectId, then somethign went wrong - we need to stop now

        If meshId = ObjectId.Null Then Exit Sub


        'Now we have our original solid and the mesh created from the cloned solid

        'Add creases
        If Not addCreases(meshId) Then Exit Sub

        'randomly extrude faces
        If Not extrudeFaces(meshId) Then Exit Sub

        'Give faces random colors
        colorFaces(meshId)

        'Convert mesh to solid
        'This is most likely place an exception will be thrown. We're not catching it.
        Dim newSolidId As ObjectId = convertToSolid(meshId)

        'Move entities so they don't sit on top of each other
        translateEntity(meshId, 1.5)
        translateEntity(newSolidId, 3)



    End Sub

End Class





