﻿// (C) Copyright 2002-2009 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//
using System;
using Autodesk.AutoCAD.Runtime;
using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.Geometry;
using Autodesk.AutoCAD.EditorInput;
using Autodesk.AutoCAD.Interop;
using Autodesk.AutoCAD.Interop.Common;
using Autodesk.AutoCAD.ThreeDimensions;


namespace Autodesk.AutoCAD.ThreeDimensions
{

  // This class is instantiated by AutoCAD for each document when
  // a command is called by the user the first time in the context
  // of a given document. In other words, non static data in this class
  // is implicitly per-document!
  public partial class MyCommands
  {
    // The CommandMethod attribute can be applied to any public  member 
    // function of any public class.
    // The function should take no arguments and return nothing.
    // If the method is an intance member then the enclosing class is 
    // intantiated for each document. If the member is a static member then
    // the enclosing class is NOT intantiated.
    //
    // NOTE: CommandMethod has overloads where you can provide helpid and
    // context menu.

    // create an ASM cylinder, taken from Kean's blog - condensed by Fenton Webb
    [CommandMethod("mycylinder")]
    public void mycylinder()
    {
      // get the usuals
      Document doc = Application.DocumentManager.MdiActiveDocument;
      Database db = doc.Database;
      Editor ed = doc.Editor;

      // Ask the user whether to create history
      bool createHistory = false;
      PromptKeywordOptions pko = new PromptKeywordOptions("Record history for this cylinder?");
      pko.AllowNone = true;
      pko.Keywords.Add("Yes");
      pko.Keywords.Add("No");
      pko.Keywords.Default = "Yes";
      PromptResult pkr = ed.GetKeywords(pko);
      // if ok
      if (pkr.Status == PromptStatus.OK)
      {
        // and he pressed Yes
        if (pkr.StringResult == "Yes")
          createHistory = true;

        // select the insertion point
        PromptPointResult ppr = ed.GetPoint("Select point");
        // if ok
        if (ppr.Status == PromptStatus.OK)
        {
          Point3d pt = ppr.Value;
          // Create the solid and set the history flag
          using (Solid3d solid = new Solid3d())
          {
            // setup the history flag as per the user request
            solid.RecordHistory = createHistory;
            // Hardcode the dimensions of the cylinder for the purpose of this example
            solid.CreateFrustum(10, 3, 3, 3);
            // Add the Solid3d to the modelspace
            using (BlockTableRecord curSpace = db.CurrentSpaceId.Open(OpenMode.ForWrite) as BlockTableRecord)
            {
              curSpace.AppendEntity(solid);
              // And transform it to the selected point
              solid.TransformBy(Matrix3d.Displacement(pt - Point3d.Origin));
            }
          }
        }
      }
    }

    // create an ASM Sphere - by Fenton Webb, DevTech, 08/27/09
    [CommandMethod("mySphere")]
    public void mySphere()
    {
      // get the usuals
      Document doc = Application.DocumentManager.MdiActiveDocument;
      Database db = doc.Database;
      Editor ed = doc.Editor;

      // Ask the user whether to create history
      bool createHistory = false;
      PromptKeywordOptions pko = new PromptKeywordOptions("Record history for this Sphere?");
      pko.AllowNone = true;
      pko.Keywords.Add("Yes");
      pko.Keywords.Add("No");
      pko.Keywords.Default = "Yes";
      PromptResult pkr = ed.GetKeywords(pko);
      // if ok
      if (pkr.Status == PromptStatus.OK)
      {
        // and he pressed Yes
        if (pkr.StringResult == "Yes")
          createHistory = true;

        // select the insertion point
        PromptPointResult ppr = ed.GetPoint("Select point");
        // if ok
        if (ppr.Status == PromptStatus.OK)
        {
          Point3d pt = ppr.Value;
          // Create the solid and set the history flag
          using (Solid3d solid = new Solid3d())
          {
            // setup the history flag as per the user request
            solid.RecordHistory = createHistory;
            // Hardcode the dimensions of the cylinder for the purpose of this example
            solid.CreateSphere(5);
            // Add the Solid3d to the modelspace
            using (BlockTableRecord curSpace = db.CurrentSpaceId.Open(OpenMode.ForWrite) as BlockTableRecord)
            {
              curSpace.AppendEntity(solid);
              // and transform it to the selected point
              solid.TransformBy(Matrix3d.Displacement(pt - Point3d.Origin));
            }
          }
        }
      }
    }

    // rotate an object by set number of degrees about axis, by Fenton Webb
    [CommandMethod("myRotate")]
    public void myRotate()
    {
      // get the usuals
      Document doc = Application.DocumentManager.MdiActiveDocument;
      Database db = doc.Database;
      Editor ed = doc.Editor;

      // pick the object to rotate
      PromptEntityResult res = ed.GetEntity("Pick entity to rotate");
      // of ok
      if (res.Status == PromptStatus.OK)
      {
        // ask how many degrees to rotate by
        PromptPointResult point = ed.GetPoint("Pick a point to Rotate about");
        // if ok
        if (point.Status == PromptStatus.OK)
        {
          // ask how many degrees to rotate by
          PromptDoubleResult rotation = ed.GetDouble("Degrees to rotate by");
          // if ok
          if (rotation.Status == PromptStatus.OK)
          {
            // finally ask which axis to rotate around
            PromptResult axis = ed.GetKeywords("Which Axis do you want to rotate around", "Xaxis Yaxis Zaxis");
            // if ok
            if (axis.Status == PromptStatus.OK)
            {
              Vector3d axisVector = Vector3d.XAxis;
              // now find out what axis he wants
              switch (axis.StringResult)
              {
                case "Yaxis": axisVector = Vector3d.YAxis; break;
                case "Zaxis": axisVector = Vector3d.ZAxis; break;
              }
              // open the entity for write
              using (Entity ent = res.ObjectId.Open(OpenMode.ForWrite) as Entity)
              {
                double pi = System.Math.Acos(-1);
                // set up a rotation matrix
                Matrix3d rotateMatrix = new Matrix3d();
                rotateMatrix = Matrix3d.Rotation(rotation.Value / 180 * pi, axisVector, point.Value);
                // now apply it to the object
                ent.TransformBy(rotateMatrix);
              }
            }
          }
        }
      }
    }

    // Union some selected objects, by Fenton Webb
    [CommandMethod("myUnion")]
    public void myUnion()
    {
      // get the usuals
      Document doc = Application.DocumentManager.MdiActiveDocument;
      Database db = doc.Database;
      Editor ed = doc.Editor;

      // select some solids to union together
      PromptSelectionOptions opts = new PromptSelectionOptions();
      opts.MessageForAdding = "Pick the solids that you want to Union";
      // setup selection filter so that only classes of type Solid3d are selected
      TypedValue[] dxfFilter = new TypedValue[1];
      // create the dxf name - now I could just have used 3DSOLID but I always 
      // get frustrated that I have to start autocad to find out, so here's code
      string dxfName = RXClass.GetClass(new Solid3d().GetType()).DxfName;
      dxfFilter[0] = new TypedValue(0, dxfName/*"3DSolid"*/); 
      SelectionFilter selectionFilter = new SelectionFilter(dxfFilter);
      // finally do the selection
      PromptSelectionResult res = ed.GetSelection(opts, selectionFilter);
      // if ok
      if (res.Status == PromptStatus.OK)
      {
        // loop each item and try to union it together
        for (int i=0; i<res.Value.Count; ++i)
        {
          ObjectId id = res.Value[i].ObjectId;
          // open the current solid for write
          using (Solid3d solid = id.Open(OpenMode.ForWrite) as Solid3d)
          {
            solid.RecordHistory = true;
            // now loop the others and try and union to it
            for (int j=i+1; j<res.Value.Count; ++j)
            {
              id = res.Value[j].ObjectId;
              // open the union-to solid for write
              using (Solid3d solidTo = id.Open(OpenMode.ForWrite) as Solid3d)
              {
                // now union them together
                solid.BooleanOperation(BooleanOperationType.BoolUnite, solidTo);
              }
            }
          }
        }
      }
    }

    // Subtract Solids from some parent host solids, by Fenton Webb
    [CommandMethod("mySubtract")]
    public void mySubtract()
    {
      // get the usuals
      Document doc = Application.DocumentManager.MdiActiveDocument;
      Database db = doc.Database;
      Editor ed = doc.Editor;

      // select some solids to subtract from
      PromptSelectionOptions opts = new PromptSelectionOptions();
      opts.MessageForAdding = "Pick Solids that you want to Subtract From";
      // setup selection filter so that only classes of type Solid3d are selected
      TypedValue[] dxfFilter = new TypedValue[1];
      // create the dxf name - now I could just have used 3DSOLID but I always 
      // get frustrated that I have to start autocad to find out, so here's code
      string dxfName = RXClass.GetClass(new Solid3d().GetType()).DxfName;
      dxfFilter[0] = new TypedValue(0, dxfName/*"3DSolid"*/);
      SelectionFilter selectionFilter = new SelectionFilter(dxfFilter);
      // finally do the selection
      PromptSelectionResult solidParentsRes = ed.GetSelection(opts, selectionFilter);
      // if ok
      if (solidParentsRes.Status == PromptStatus.OK)
      {
        // select some solids to subtract
        opts.MessageForAdding = "Now the Solids that you want to Subtract";
        // finally do the selection
        PromptSelectionResult subtractRes = ed.GetSelection(opts, selectionFilter);
        // if ok
        if (subtractRes.Status == PromptStatus.OK)
        {
          // loop each item and try to subtract them together
          for (int i = 0; i < solidParentsRes.Value.Count; ++i)
          {
            ObjectId id = solidParentsRes.Value[i].ObjectId;
            // open the current solid for write
            using (Solid3d solid = id.Open(OpenMode.ForWrite) as Solid3d)
            {
              solid.RecordHistory = true;
              // now loop the others and try to subtract them
              for (int j = 0; j < subtractRes.Value.Count; ++j)
              {
                id = subtractRes.Value[j].ObjectId;
                // open the union-to solid for write
                using (Solid3d solidTo = id.Open(OpenMode.ForWrite) as Solid3d)
                {
                  solidTo.RecordHistory = true;
                  // ok ready to subtract them from each other
                  solid.BooleanOperation(BooleanOperationType.BoolSubtract, solidTo);
                }
              }
            }
          }
        }
      }
    }

    // Intersect selected objects producing a solid consisting of everything
    // that intersects, by Fenton Webb
    [CommandMethod("myIntersect")]
    public void myIntersect()
    {
      // get the usuals
      Document doc = Application.DocumentManager.MdiActiveDocument;
      Database db = doc.Database;
      Editor ed = doc.Editor;

      // select some solids to union together
      PromptSelectionOptions opts = new PromptSelectionOptions();
      opts.MessageForAdding = "Pick the solids that you want to Intersect";
      // setup selection filter so that only classes of type Solid3d are selected
      TypedValue[] dxfFilter = new TypedValue[1];
      // create the dxf name - now I could just have used 3DSOLID but I always 
      // get frustrated that I have to start autocad to find out, so here's code
      string dxfName = RXClass.GetClass(new Solid3d().GetType()).DxfName;
      dxfFilter[0] = new TypedValue(0, dxfName/*"3DSolid"*/);
      SelectionFilter selectionFilter = new SelectionFilter(dxfFilter);
      // finally do the selection
      PromptSelectionResult res = ed.GetSelection(opts, selectionFilter);
      // if ok
      if (res.Status == PromptStatus.OK)
      {
        // loop each item and try to union it together
        for (int i = 0; i < res.Value.Count; ++i)
        {
          ObjectId id = res.Value[i].ObjectId;
          // open the current solid for write
          using (Solid3d solid = id.Open(OpenMode.ForWrite) as Solid3d)
          {
            // now loop the others and try and union to it
            for (int j = i + 1; j < res.Value.Count; ++j)
            {
              id = res.Value[j].ObjectId;
              // open the union-to solid for write
              using (Solid3d solidTo = id.Open(OpenMode.ForWrite) as Solid3d)
              {
                solid.RecordHistory = true;
                // now union them together
                solid.BooleanOperation(BooleanOperationType.BoolIntersect, solidTo);
              }
            }
          }
        }
      }
    }

    [CommandMethod("editMyCylinder")]
    public void editMyCylinder()
    {
      // get the usuals
      Document doc = Application.DocumentManager.MdiActiveDocument;
      Database db = doc.Database;
      Editor ed = doc.Editor;

      // now select the cylinder for editing
      PromptEntityOptions opts = new PromptEntityOptions("Pick the cylinder to edit");
      // now select one
      PromptEntityResult res = ed.GetEntity(opts);
      // if ok
      if (res.Status == PromptStatus.OK)
      {
        // open the object for read and try and cast it to a Solid3d
        try
        {
          // try and cast it to a solid3d
          using (Solid3d solid = res.ObjectId.Open(OpenMode.ForRead) as Solid3d)
          {
            // get the AutoCAD ActiveX application object
            AcadApplication acadApp = Application.AcadApplication as AcadApplication;
            // if it is a solid3d, next get its COM interface because that will provide
            // use with some useful property accessors
            Acad3DSolid acadSolid = acadApp.ActiveDocument.ObjectIdToObject(solid.ObjectId.OldId) as Acad3DSolid;
            // now set some properties            
            ed.WriteMessage("\nSolid Type = " + acadSolid.SolidType.ToString());

            // get the rx class entry
            RXClass pAcrxClass = solid.GetRXClass();
            // Per-instance dynamic property managers
            //OPMPerInstancePropertyExtensionFactory perInstanceExtension = OPMPerInstancePropertyExtensionFactory.cast(db.GetRXClass().QueryX(OPMPerInstancePropertyExtensionFactory.desc()))(pAcrxClass);
            //if (perInstanceExtension != null)
            {

            }
          }
        }
        catch
        {
          ed.WriteMessage("\nNot a Solid3d...");
        }          
      }
    }
  }
}
