﻿// (C) Copyright 2002-2009 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//
using System;
using Autodesk.AutoCAD.Runtime;
using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.Geometry;
using Autodesk.AutoCAD.EditorInput;

// This line is not mandatory, but improves loading performances
[assembly: CommandClass(typeof(Autodesk.AutoCAD.ThreeDimensions.MyCommands))]

namespace Autodesk.AutoCAD.ThreeDimensions
{
  // This class is instantiated by AutoCAD for each document when
  // a command is called by the user the first time in the context
  // of a given document. In other words, non static data in this class
  // is implicitly per-document!
  public partial class MyCommands
  {
    // The CommandMethod attribute can be applied to any public  member 
    // function of any public class.
    // The function should take no arguments and return nothing.
    // If the method is an instance member then the enclosing class is 
    // intantiated for each document. If the member is a static member then
    // the enclosing class is NOT intsantiated.
    //
    // NOTE: CommandMethod has overloads where you can provide helpid and
    // context menu.
    [CommandMethod("loftGeometry")]
    public void loftGeometry()
    {
      // get the normal vars
      Document doc = Application.DocumentManager.MdiActiveDocument;
      Database db = doc.Database;
      Editor ed = doc.Editor;

      // pick a point to place this
      PromptPointOptions opts = new PromptPointOptions("Pick insertion point");
      PromptPointResult res = ed.GetPoint(opts);
      // if ok
      if (res.Status == PromptStatus.OK)
      {
        // extract the pick point into a 2d point
        Point2d pnt = new Point2d(res.Value.X, res.Value.Y);

        // create somethings to loft
        using (Polyline star1 = Utils.CreateStarOutline(pnt, 0))
        using (Polyline star2 = Utils.CreateStarOutline(pnt, 10))
        using (Polyline star3 = Utils.CreateStarOutline(pnt, 30))
        {
          // now lets move star2 and star3 into place
          Matrix3d matrix = new Matrix3d();
          // move star2 in the Z
          matrix = Matrix3d.Displacement(new Vector3d(0, 0, 20));
          star2.TransformBy(matrix);
          // move star3 in the Z
          matrix = Matrix3d.Displacement(new Vector3d(0, 0, 55));
          star3.TransformBy(matrix);
          // next lets give star2 a little more size
          matrix = Matrix3d.Scaling(1.2, star2.GetPoint3dAt(0));
          star2.TransformBy(matrix);
          // add the result to the current space
          using (BlockTableRecord curSpace = db.CurrentSpaceId.Open(OpenMode.ForWrite) as BlockTableRecord)
          {
            curSpace.AppendEntity(star1);
            curSpace.AppendEntity(star2);
            curSpace.AppendEntity(star3);
          }
        }
      }
    }

    // loft to cross sections
    [CommandMethod("myLoft")]
    public void myLoft()
    {
      // get the normal vars
      Document doc = Application.DocumentManager.MdiActiveDocument;
      Database db = doc.Database;
      Editor ed = doc.Editor;

      PromptSelectionOptions opts = new PromptSelectionOptions();
      opts.MessageForAdding = "Objects to loft";
      // setup selection filter so that only classes of type Solid3d are selected
      TypedValue[] dxfFilter = new TypedValue[1];
      // create the dxf name
      string dxfName = RXClass.GetClass(new Polyline().GetType()).DxfName;
      dxfFilter[0] = new TypedValue(0, dxfName/*"POLYLINE"*/); 
      SelectionFilter selectionFilter = new SelectionFilter(dxfFilter);
      // finally do the selection
      PromptSelectionResult res = ed.GetSelection(opts, selectionFilter);
      // if ok
      if (res.Status == PromptStatus.OK)
      {
        // set up the loft properties
        LoftOptions loftOptions = new LoftOptions();
        // to set the values we need to create a builder
        LoftOptionsBuilder builder = new LoftOptionsBuilder(loftOptions);

        // start a transaction this time, we have to open all the selected items for read
        using (Transaction trans = db.TransactionManager.StartOpenCloseTransaction())
        {
          // set the profile objects
          Entity[] ents = new Entity[res.Value.Count];
          // loop the selection
          for (int i = 0; i < res.Value.Count; ++i)
          {
            ObjectId id = res.Value[i].ObjectId;
            ents[i] = trans.GetObject(id, OpenMode.ForRead) as Entity;
          }

          // even if we don't use guides it must be set to a zero length array
          Entity[] guides = new Entity[0];

          // now create a new solid, you can mix transactions with 
          using (Solid3d solid = new Solid3d())
          {
            solid.SetDatabaseDefaults();
            // now we are ready to create the lofted solid
            solid.CreateLoftedSolid(ents, guides, null, loftOptions);
            // add the result to the current space
            using (BlockTableRecord curSpace = db.CurrentSpaceId.Open(OpenMode.ForWrite) as BlockTableRecord)
            {
              curSpace.AppendEntity(solid);
              trans.Commit();
            }
          }
        }
      }        
    }

    // loft to cross sections
    [CommandMethod("myLoftOptions")]
    public void myLoftOptions()
    {
      // get the normal vars
      Document doc = Application.DocumentManager.MdiActiveDocument;
      Database db = doc.Database;
      Editor ed = doc.Editor;

      PromptSelectionOptions opts = new PromptSelectionOptions();
      opts.MessageForAdding = "Objects to loft";
      // setup selection filter so that only classes of type Solid3d are selected
      TypedValue[] dxfFilter = new TypedValue[1];
      // create the dxf name
      string dxfName = RXClass.GetClass(new Polyline().GetType()).DxfName;
      dxfFilter[0] = new TypedValue(0, dxfName/*"POLYLINE"*/);
      SelectionFilter selectionFilter = new SelectionFilter(dxfFilter);
      // finally do the selection
      PromptSelectionResult res = ed.GetSelection(opts, selectionFilter);
      // if ok
      if (res.Status == PromptStatus.OK)
      {
        // set up the loft properties
        LoftOptions loftOptions = new LoftOptions();
        // to set the values we need to create a builder
        LoftOptionsBuilder builder = new LoftOptionsBuilder(loftOptions);

        // setup the options for the Keyword input
        PromptKeywordOptions opts1 = new PromptKeywordOptions("Enter LoftOptions [Smoothfit/Ruled/Normalto]", "Smoothfit Ruled Normalto");
        opts1.Keywords.Default = "Smootfit";
        // now get the input from the user via the commandline
        PromptResult res1 = ed.GetKeywords(opts1);
        // else if enter is pressed
        // now check the result, if ok
        if (res1.Status == PromptStatus.OK)
        {
          // if the keyword typed is Smoothfit
          if (res1.StringResult == "Smoothfit")
          {
            // do nothing as that is what it will do anyway
          }
          // if the keyword typed is Ruled
          else if (res1.StringResult == "Ruled")
            builder.Ruled = true;
          // if the keyword typed is Normalto
          else if (res1.StringResult == "Normalto")
          {
            // setup the options for the Keyword input
            PromptKeywordOptions opts2 = new PromptKeywordOptions("Select Normal Options [Allnormal/Endsnormal/Firstnormal/Lastnormal/Nonormal/Usedraftangles]", "Allnormal Endsnormal Firstnormal Lastnormal Nonormal Usedraftangles");
            opts2.Keywords.Default = "AllNormal";
            // now get the input from the user via the commandline
            PromptResult res2 = ed.GetKeywords(opts2);
            // else if a keyword
            if (res2.Status == PromptStatus.OK)
            {
              // if the keyword typed is AllNormal
              if (res2.StringResult == "AllNormal")
                builder.NormalOption = LoftOptionsNormalOption.AllNormal;
              // if the keyword typed is EndsNormal
              else if (res2.StringResult == "Endsnormal")
                builder.NormalOption = LoftOptionsNormalOption.EndsNormal;
              // if the keyword typed is FirstNormal
              else if (res2.StringResult == "Firstnormal")
                builder.NormalOption = LoftOptionsNormalOption.FirstNormal;
              // if the keyword typed is LastNormal
              else if (res2.StringResult == "Lastnormal")
                builder.NormalOption = LoftOptionsNormalOption.LastNormal;
              // if the keyword typed is NoNormal
              else if (res2.StringResult == "Nonormal")
                builder.NormalOption = LoftOptionsNormalOption.NoNormal;
              // if the keyword typed is UseDraftAngles
              else if (res2.StringResult == "Usedraftangles")
              {
                // setup the options for the Angle input
                PromptAngleOptions opts3 = new PromptAngleOptions("Enter Start Angle");
                opts3.DefaultValue = 0;
                opts3.AllowNone = true;
                // now get the input from the user via the commandline
                PromptDoubleResult res3 = ed.GetAngle(opts3);
                // now check the result, if ok
                if (res3.Status == PromptStatus.OK)
                {
                  builder.DraftStart = res3.Value;

                  // setup the options for the Angle input
                  opts3 = new PromptAngleOptions("End Angle", "");
                  opts3.DefaultValue = 0;
                  opts3.AllowNone = true;

                  // now get the input from the user via the commandline
                  res3 = ed.GetAngle(opts3);
                  // now check the result, if ok
                  if (res3.Status == PromptStatus.OK)
                  {
                    builder.DraftStartMag = res3.Value;
                    // setup the options for the Distance input
                    PromptDistanceOptions opts4 = new PromptDistanceOptions("Start Magnitude", "");
                    opts4.DefaultValue = 0;
                    opts4.AllowNone = true;

                    // now get the input from the user via the commandline
                    res3 = ed.GetDistance(opts4);
                    // now check the result, if ok
                    if (res3.Status == PromptStatus.OK)
                    {
                      builder.DraftEnd = res3.Value;

                      // setup the options for the Distance input
                      opts4 = new PromptDistanceOptions("End Magnitude");
                      opts4.DefaultValue = 0;
                      opts4.AllowNone = true;
  
                      // now get the input from the user via the commandline
                      res3 = ed.GetDistance(opts4);
                      // now check the result, if ok
                      if (res3.Status == PromptStatus.OK)
                        builder.DraftEndMag = res3.Value;                
                    }
                  }
                }
              }
            }
          }
        }

        // start a transaction this time, we have to open all the selected items for read
        using (Transaction trans = db.TransactionManager.StartTransaction())
        {
          // set the profile objects
          Entity[] ents = new Entity[res.Value.Count];
          // loop the selection
          for (int i = 0; i < res.Value.Count; ++i)
          {
            ObjectId id = res.Value[i].ObjectId;
            ents[i] = trans.GetObject(id, OpenMode.ForRead) as Entity;
          }

          // even if we don't use guides it must be set to a zero length array
          Entity[] guides = new Entity[0];

          // now create a new solid, you can mix transactions with 
          using (Solid3d solid = new Solid3d())
          {
            solid.SetDatabaseDefaults();
            // now we are ready to create the lofted solid
            solid.CreateLoftedSolid(ents, guides, null, loftOptions);
            // add the result to the current space
            using (BlockTableRecord curSpace = db.CurrentSpaceId.Open(OpenMode.ForWrite) as BlockTableRecord)
            {
              curSpace.AppendEntity(solid);
              trans.Commit();
            }
          }
        }
      }
    }

    // create a lamp with 2 guides mirrored n each side
    [CommandMethod("lamp2")]
    public void lamp2()
    {
      // get the normal vars
      Document doc = Application.DocumentManager.MdiActiveDocument;
      Database db = doc.Database;
      Editor ed = doc.Editor;

      // now create some geometry
      Circle circle1 = new Circle(new Point3d(18.0, 8.0, 15.5), Vector3d.ZAxis, 2.0);
      Circle circle2 = new Circle(new Point3d(18.0, 8.0, 17.5), Vector3d.ZAxis, 1.0);

      // create a path for the loft to follow on the right
      Point3dCollection pts = new Point3dCollection();
      pts.Add(new Point3d(20.0, 8.0, 15.5));
      pts.Add(new Point3d(19.3, 8.0, 16.5));
      pts.Add(new Point3d(19.0, 8.0, 17.5));
      // In memory splines that serve as guide curves for the lamp shade
      Spline spline1 = new Spline(pts, 4, 0);
      spline1.SetDatabaseDefaults();

      // create a path for the loft to follow on the left
      Point3dCollection pts2 = new Point3dCollection();
      pts2.Add(new Point3d(16.0, 8.0, 15.5));
      pts2.Add(new Point3d(16.7, 8.0, 16.5));
      pts2.Add(new Point3d(17.0, 8.0, 17.5));
      // In memory splines that serve as guide curves for the lamp shade
      Spline spline2 = new Spline(pts2, 4, 0);
      spline2.SetDatabaseDefaults();

      // now create a new solid
      using (Solid3d solid = new Solid3d())
      {
        // set up the loft properties, this is all read only and will be set when finished
        LoftOptions loftOptions = new LoftOptions();
        // set the 2 start and end profile objects
        Entity[] ents = new Entity[1];
        ents[0] = circle1;
        ents[1] = circle2;
        // set the guides or paths for the loft to follow
        Spline[] guides = new Spline[1];
        guides[0] = spline1;
        guides[1] = spline2;
        // now we are ready to create the lofted solid
        solid.CreateLoftedSolid(ents, guides, null, loftOptions);

        // add the result to the current space
        using (BlockTableRecord curSpace = db.CurrentSpaceId.Open(OpenMode.ForWrite) as BlockTableRecord)
        {
          curSpace.AppendEntity(solid);
          //curSpace.AppendEntity(circle1)
          //curSpace.AppendEntity(circle2)
          //curSpace.AppendEntity(spline1)
          //curSpace.AppendEntity(spline2)
          //circle1.Close()
          //circle2.Close()
          //spline1.Close()
          //spline2.Close()
        }
      }
    }
    // same lamp but with just one guide or path
    [CommandMethod("lamp1")]
    public void lamp1()
    {
      // get the normal vars
      Document doc = Application.DocumentManager.MdiActiveDocument;
      Database db = doc.Database;
      Editor ed = doc.Editor;

      // now create some geometry
      Circle circle1 = new Circle(new Point3d(18.0, 8.0, 15.5), Vector3d.ZAxis, 2.0);
      Circle circle2 = new Circle(new Point3d(18.0, 8.0, 17.5), Vector3d.ZAxis, 1.0);

      // create a path for the loft to follow on the right
      Point3dCollection pts = new Point3dCollection();
      pts.Add(new Point3d(20.0, 8.0, 15.5));
      pts.Add(new Point3d(19.3, 8.0, 16.5));
      pts.Add(new Point3d(19.0, 8.0, 17.5));
      // In memory splines that serve as guide curves for the lamp shade
      Spline spline1 = new Spline(pts, 4, 0);
      spline1.SetDatabaseDefaults();

      // create a path for the loft to follow on the left
      Point3dCollection pts2 = new Point3dCollection();
      pts2.Add(new Point3d(16.0, 8.0, 15.5));
      pts2.Add(new Point3d(16.7, 8.0, 16.5));
      pts2.Add(new Point3d(17.0, 8.0, 17.5));
      // In memory splines that serve as guide curves for the lamp shade
      Spline spline2 = new Spline(pts2, 4, 0);
      spline2.SetDatabaseDefaults();
      
      // now create a new solid
      using (Solid3d solid = new Solid3d())
      {
        // set up the loft properties, this is all read only and will be set when finished
        LoftOptions loftOptions = new LoftOptions();
        // set the 2 start and end profile objects
        Entity[] ents = new Entity[1];
        ents[0] = circle1;
        ents[1] = circle2;
        // set the guides or paths for the loft to follow
        Spline[] guides = new Spline[0];
        guides[0] = spline1; 
        //guides[1] = spline2;
        // now we are ready to create the lofted solid
        solid.CreateLoftedSolid(ents, guides, null, loftOptions);

        // add the result to the current space
        using (BlockTableRecord curSpace = db.CurrentSpaceId.Open(OpenMode.ForWrite) as BlockTableRecord)
        {
          curSpace.AppendEntity(solid);
          //curSpace.AppendEntity(circle1)
          //curSpace.AppendEntity(circle2)
          //curSpace.AppendEntity(spline1)
          //circle1.Close()
          //circle2.Close()
          //spline1.Close()
        }
      }
    }
  }
}
